@@ -137,7 +137,7 @@ void log_close( void )
 void log_msg( char *msg, ... )
 {
 
-	char    *format_temp;
+	char    *format_temp = NULL;
 	va_list arglist;
 	int     length;        /* length of msg */
 	int     retc;          /* return code */
@@ -191,5 +191,6 @@ void log_msg( char *msg, ... )
 		va_end( arglist );
 	}
 	syslog((facility | level), "%s", format_temp);
+	free(format_temp);
 }
 
