@@ -844,7 +844,7 @@ int process_user( ShellOptions_t *opts,
 	char	user[CFG_LINE_LEN + 1];	/* username of user entry */
 	char	mask[CFG_LINE_LEN + 1]; /* user's umask */
 	char	axs[CFG_LINE_LEN + 1];	/* user's access bits */
-	char	*path;			/* user's chroot path */
+	char	*path = NULL;		/* user's chroot path */
 	int	tmpmask;		/* temporary umask holder */
 	int	pos;			/* count into line */
 	int	len;			/* add len of token to pos */
@@ -861,7 +861,6 @@ int process_user( ShellOptions_t *opts,
 		exit(1);
 	}
 
-	/* raise(SIGSTOP); */
 	/* get the config bits and eat comments */
 	if ( !get_asgn_param(line, lineno, temp, CFG_LINE_LEN) ){
 		free(temp);
