@@ -106,7 +106,7 @@ void fail( int flags, int argc, char **argv )
 	/* print error message to user and log attempt */
 	fprintf(stderr, "\nThis account is restricted by rssh.\n"
 		"%s\n\nIf you believe this is in error, please contact "
-	        "your system administrator.\n\n", cmd);
+		"your system administrator.\n\n", cmd);
 	if ( argc < 3 )
 		log_msg("user %s attempted to log in with a shell",
 			username);
@@ -132,123 +132,144 @@ void fail( int flags, int argc, char **argv )
  */
 bool opt_exist(char *cl, char opt)
 {
-	int	i = 0;
+	int	i = 1;
 	int	len;
-	char	*token;
-	bool	optstring = FALSE;
-
 
 	len = strlen(cl);
 
 	/* process command line character by character */
-	while ( i < (len - 2) ){
-		if ( cl[i] == ' ' || cl[i] == '\t' ){
-			if ( cl[i+1] == '-' ){ 
-				optstring = TRUE;
-				i+=2;
-			}
-		}
-		if ( cl[i] == opt && optstring ) return TRUE;
-		if ( cl[i] == ' ' || cl[i] == '\t' || cl[i] == '-' ) 
-			optstring = FALSE;
+	if (!(cl[0] == '-')) return FALSE;
+	while ( i < (len) ){
+		if ( cl[i] == opt ) return TRUE;
 		i++;
 	}
 	return FALSE;
 }
 
 
+bool opt_filter(char **vec, const char opt)
+{
+	while (vec && *vec){
+		if (opt_exist(*vec, opt)){
+			fprintf(stderr, "\nillegal insecure %c option", opt);
+			log_msg("insecure %c option in scp command line!", opt);
+			return TRUE;
+		}
+		vec++;
+	}
+	return FALSE;
+}
+
+
 bool check_command( char *cl, ShellOptions_t *opts, char *cmd, int cmdflag )
 {
-	int	cl_len;		/* length of command line */
-	int	len;		/* length of allowed command */
-	int	altlen;		/* length of allowed command without path */
-	char	*progname;	/* basename of program */
+	char	*prog;		/* basename of cmd */
+	char 	*tmp = cl;
+	bool	need_free = FALSE;
+	bool	rc = FALSE;
+	int 	i;
+	size_t	len;
+
 
 	if ( opts->shell_flags & cmdflag ){
+		/* cl may be a full command line; if so get first word */
+		len = strlen(cl);
+		for (i=0; i < len && !isspace(*(cl + i)); i++);
+		if (i < len){
+			if (!(tmp = (char *)malloc(sizeof (char) * (i+1)))){
+				log_msg("malloc() failed in check_command()");
+				return FALSE;
+			}
+			need_free = TRUE;
+			memcpy(tmp, cl, i);
+			tmp[i] = '\0';
+		}
 
-		cl_len = strlen(cl);
-		len = strlen(cmd);
-		progname = basename(cmd);
-		altlen = strlen(progname);
-
-		/* make sure we don't compare more of the string than exists */
-		if ( cl_len < len ) len = cl_len;
-		if ( cl_len < altlen ) altlen = cl_len;
-
-		/* compare for match */
-		if ( (!(strncmp(cl, cmd, len)) && 
-					((isspace(cl[len]) || cl[len] == '\0')))
-		     || (!(strncmp(cl, progname, altlen)) && 
-					((isspace(progname[altlen]) || 
-					 progname[altlen] == '\0'))) )
-			return TRUE;
+		/* compare tmp to cmd and prog for match */
+		prog = basename(cmd);
+		if ( !(strcmp(tmp, cmd)) || !(strcmp(tmp, prog))){
+			log_msg("cmd '%s' approved", prog);
+			rc = TRUE;
+		}	
 	}
-	return FALSE;
+	if (need_free) free(tmp);
+	return rc;
 }
 
+
 /*
  * check_command_line() - take the command line passed to rssh, and verify
- * 			  that the specified command is one the user is
- * 			  allowed to run.  Return the path of the command
- * 			  which will be run if it is ok, or return NULL if it
- * 			  is not.
+ *			  that the specified command is one the user is
+ *			  allowed to run and validate the arguments.  Return the
+ *			  path of the command which will be run if it is ok, or
+ *			  return NULL if it is not.
  */
-char *check_command_line( char *cl, ShellOptions_t *opts )
+char *check_command_line( char **cl, ShellOptions_t *opts )
+{
+
+	/* Here we call check_command not to validate, but to match */
+	if ( check_command(*cl, opts, PATH_SFTP_SERVER, RSSH_ALLOW_SFTP) )
+		return PATH_SFTP_SERVER;
+
+	if ( check_command(*cl, opts, PATH_SCP, RSSH_ALLOW_SCP) ){
+		/* filter -S option */
+		if ( opt_filter(cl, 'S') ) return NULL;
+		return PATH_SCP;
+	}
+
+	if ( check_command(*cl, opts, PATH_CVS, RSSH_ALLOW_CVS) ){
+		if ( opt_filter(cl, 'e') ) return NULL;
+		return PATH_CVS;
+	}
+
+	if ( check_command(*cl, opts, PATH_RDIST, RSSH_ALLOW_RDIST) ){
+		/* filter -P option */
+		if ( opt_filter(cl, 'P') ) return NULL;
+		return PATH_RDIST;
+	}
+
+	if ( check_command(*cl, opts, PATH_RSYNC, RSSH_ALLOW_RSYNC) ){
+		/* filter -e option */
+		if ( opt_filter(cl, 'e') ) return NULL;
+		while (cl && *cl){
+			if ( strstr(*cl, "--rsh" ) ){
+				fprintf(stderr, "\ninsecure --rsh= not allowed.");
+				log_msg("insecure --rsh option in rsync command line!");
+				return NULL;
+			}
+			cl++;
+		}
+		return PATH_RSYNC;
+	}
+	/* No match, return NULL */
+	return NULL;
+}
+
+
+/*
+ * get_command() - take the command line passed to rssh, and verify
+ *		   that the specified command is one the user is allowed to run.
+ *		   Return the path of the command which will be run if it is ok,
+ *		   or return NULL if it is not.
+ */
+char *get_command( char *cl, ShellOptions_t *opts )
 {
 
 	if ( check_command(cl, opts, PATH_SFTP_SERVER, RSSH_ALLOW_SFTP) )
 		return PATH_SFTP_SERVER;
-
-	if ( check_command(cl, opts, PATH_SCP, RSSH_ALLOW_SCP) ){
-		/* filter -S option */
-		if ( opt_exist(cl, 'S') ){
-			fprintf(stderr, "\ninsecure -S option not allowed.");
-			log_msg("insecure -S option in scp command line!");
-			return NULL;
-		}
+	if ( check_command(cl, opts, PATH_SCP, RSSH_ALLOW_SCP) )
 		return PATH_SCP;
-	}
-
-	if ( check_command(cl, opts, PATH_CVS, RSSH_ALLOW_CVS) ){
-		if ( opt_exist(cl, 'e') ){
-			fprintf(stderr, "\ninsecure -e option not allowed.");
-			log_msg("insecure -e option in cvs command line!");
-			return NULL;
-		}
+	if ( check_command(cl, opts, PATH_CVS, RSSH_ALLOW_CVS) )
 		return PATH_CVS;
-	}
-
-	if ( check_command(cl, opts, PATH_RDIST, RSSH_ALLOW_RDIST) ){
-		/* filter -P option */
-		if ( opt_exist(cl, 'P') ){
-			fprintf(stderr, "\ninsecure -P option not allowed.");
-			log_msg("insecure -P option in rdist command line!");
-			return NULL;
-		}
+	if ( check_command(cl, opts, PATH_RDIST, RSSH_ALLOW_RDIST) )
 		return PATH_RDIST;
-	}
-
-	if ( check_command(cl, opts, PATH_RSYNC, RSSH_ALLOW_RSYNC) ){
-		/* filter -e option */
-		if ( opt_exist(cl, 'e') ){
-			fprintf(stderr, "\ninsecure -e option not allowed.");
-			log_msg("insecure -e option in rdist command line!");
-			return NULL;
-		}
-		
-		if ( strstr(cl, "--rsh=" ) ){
-			fprintf(stderr, "\ninsecure --rsh= not allowed.");
-			log_msg("insecure --rsh option in rsync command line!");
-			return NULL;
-		}
-
+	if ( check_command(cl, opts, PATH_RSYNC, RSSH_ALLOW_RSYNC) )
 		return PATH_RSYNC;
-	}
-
 	return NULL;
 }
 
 
+
 /*
  * extract_root() - takes a root directory and the full path to some other
  *                  directory, and returns a pointer to a string which
@@ -264,7 +285,7 @@ char *extract_root( char *root, char *path )
 	len = strlen(root);
 	/* get rid of a trailing / from the root path */
 	if ( root[len - 1] == '/' ){
-	       	root[len - 1] = '\0';
+		root[len - 1] = '\0';
 		len--;
 	}
 	if ( (strncmp(root, path, len)) ) return NULL;
@@ -309,7 +330,7 @@ int validate_umask( const char *temp, int *mask )
  *                     same name, and returns FALSE if the bits are not valid
  */
 int validate_access( const char *temp, bool *allow_sftp, bool *allow_scp,
-	       	     bool *allow_cvs, bool *allow_rdist, bool *allow_rsync )
+		     bool *allow_cvs, bool *allow_rdist, bool *allow_rsync )
 {
 	int	i;
 
